function [ num_freq_out, err_rel, err_rel_trunc, err_rel_alias, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times ] = test3_f9_sfft_cl_algorithm_singletest( N, algorithm_name, threshold_theta, threshold_theta_basic, niter_r )
%TEST3_SFFT_CL_ALGORITHM_SINGLETEST Performs a single test run of sfft_algorithm on 4d function.
%
%   Input parameters:
%      N - search space parameter N, search space Gamma = [0,..,N]^d
%      algorithm_name - type of algorithm,
%                       either 'a1cl' for Algorithm 1
%                       or 'a2cl' for Algorithm 2
%      threshold_theta - relative threshold theta used for
%                        truncation in step 2e, default is 1e-12
%      threshold_theta_basic - relative threshold theta used for
%                              truncation in steps 1 and 2a,
%                              uses the value of threshold_theta as default 
%      niter_r - number of detection iterations r
%
%   Output parameters:
%      num_freq_out - number of frequencies returned by the algorithm
%      err_rel - maximal relative approximation error
%                   (contains truncation error and aliasing error)
%      err_rel_trunc - maximal relative truncation error
%      err_rel_alias - maximal relative aliasing error
%      freq_cand_sizes - array of length d with number of investigated
%                        frequencies in each dimension increment step t
%      lattice_sizes - array of length d with size M of reconstructing
%                      rank-1 lattices used for the sampling
%                      in each dimension increment step t
%      sample_sizes - array of length d with number of samples taken from
%                     the function fct_handle in each dimension increment
%                     step t
%      z - generating vector z of reconstructing rank-1 Chebyshev lattice for
%          frequencies in freq, lattice_sizes(end) contains the
%          corresponding rank-1 Chebyshev lattice size parameter M
%      run_times - array of length 5 with runtimes measured for various
%                  parts of the algorithm,
%                  run_times(1): steps 1 and 2a,
%                  run_times(2): building of reconstructing rank-1 Chebyshev
%                                lattices
%                  run_times(3): sampling step 2c
%                  run_times(4): inverse rank-1 Chebyshev lattice FFT step 2d
%                  run_times(5): total runtime of the algorithm
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

d = 9;

sparsity_s = intmax;
sparsity_s_local = intmax;
niter_r_basic = niter_r;

[ freq, fhat_out, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times ] = sfft_cl_algorithm( @bsplinet_test_9d, d, N, algorithm_name, sparsity_s, sparsity_s_local, threshold_theta, threshold_theta_basic, niter_r, niter_r_basic );

[fhat, norm_fct_square] = bsplinet_test_9d_chat(freq);
norm_fct = sqrt(norm_fct_square);

num_freq_out = size(freq,1);

err_rel_trunc = sqrt(norm_fct_square - pi^d * sum(abs(fhat).^2 ./ 2.^(sum(freq~=0,2)))) / norm_fct;
if ~isreal(err_rel_trunc)
  error('imag');
end
fprintf('rel. truncation err.: %.3e, ', err_rel_trunc);
err_rel_alias = sqrt(pi^d * sum(abs(fhat_out-fhat).^2 ./ 2.^(sum(freq~=0,2)))) / norm_fct;
fprintf('rel. aliasing err.: %.3e\n', err_rel_alias);

%err_l2_rel = sqrt( norm_fct_square - norm(fhat,2)^2 + norm(fhat_out-fhat,2)^2 ) / sqrt(norm_fct_square);
err_rel = sqrt(norm_fct_square - sum(pi^d * abs(fhat).^2 ./ 2.^(sum(freq~=0,2))) + sum(pi^d * abs(fhat_out-fhat).^2 ./ 2.^(sum(freq~=0,2)))) / norm_fct;
if ~isreal(err_rel)
  error('imag');
end
fprintf('relative Cheb approximation error: %3e\n', err_rel);

end

